#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <dirent.h>
#include <string.h>

#define checkError(expr,userMsg)\
    do {\
        if (!(expr)) {\
            perror(userMsg);\
            exit(EXIT_FAILURE);\
        } \
    } while (0)

void traverseDir(char* fPath, int* pSize) {

    struct stat fInfo;
    checkError(lstat(fPath, &fInfo) != -1, "lstat");

    if (!S_ISDIR(fInfo.st_mode)) {
        *pSize += fInfo.st_size;
        return;
    }

    DIR* dir = opendir(fPath);
    checkError(dir != NULL, "opendir");

    struct dirent* dirEnt = NULL;
    while ((dirEnt = readdir(dir)) != NULL) {

        int n = strlen(fPath) + strlen(dirEnt->d_name) + 2;
        char* newPath = malloc(n*sizeof(char));
        checkError(newPath != NULL, "malloc");

        sprintf(newPath, "%s/%s", fPath, dirEnt->d_name);

        if (strcmp(dirEnt->d_name, ".") == 0 || 
                strcmp(dirEnt->d_name, "..") == 0) {
            
            checkError(stat(newPath, &fInfo) != -1, "stat");
            *pSize += fInfo.st_size;
            free(newPath);
            continue;
        }

        traverseDir(newPath, pSize);
        free(newPath);
    } 

    checkError(closedir(dir) != -1, "closedir");
}

int main(int argc, char** argv) {

    checkError(argc == 2, "args");

    struct stat fInfo;
    checkError(stat(argv[1], &fInfo) != -1, "stat");

    checkError(S_ISDIR(fInfo.st_mode), "not a dir");

    int size = 0;
    traverseDir(argv[1], &size);

    printf("Dir size: %dB\n", size);

    exit(EXIT_SUCCESS);
}